from ngsolve import *
from netgen.geom2d import SplineGeometry
geo = SplineGeometry()
geo.AddRectangle( (0, 0), (2, 0.41), bcs = ("wall", "outlet", "wall", "inlet"))
mesh = Mesh( geo.GenerateMesh(maxh=0.05))

V = VectorH1(mesh, order=3, dirichlet="wall|outlet|inlet")
Q = H1(mesh, order=2)
N = NumberSpace(mesh)
X = FESpace([V,Q,N])

u,p,mu = X.TrialFunction()
v,q,lam = X.TestFunction()
    
a = BilinearForm(X) 
a += SymbolicBFI(InnerProduct(grad(u),grad(v))+div(u)*q+div(v)*p +mu*q + lam*p)
a.Assemble() 
    
gfu = GridFunction(X)
gfu.components[0].Set((1,1)) 
gfu.components[1].Set(1)
fdofs = X.FreeDofs()
for i in range(len(gfu.vec)):
    if fdofs[i] == False:
        gfu.vec[i] = 0
print("gfu.vec ", min(gfu.vec), max(gfu.vec))
    
t1 = gfu.vec.CreateVector()
t2 = GridFunction(X)
t1.data = a.mat * gfu.vec
t2.vec.data = a.mat.Inverse(freedofs=X.FreeDofs(), inverse="umfpack") * t1
print("t2 ", min(t2.vec), max(t2.vec))
